import RPi.GPIO as GPIO
from time import sleep


class ShiftRegister:
    def __init__(self, data_pin, latch_pin, clock_pin):
        self.data_pin = data_pin
        self.latch_pin = latch_pin
        self.clock_pin = clock_pin
        self.current_byte = 0  # Tracks the current state of the shift register

        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.data_pin, GPIO.OUT)
        GPIO.setup(self.latch_pin, GPIO.OUT)
        GPIO.setup(self.clock_pin, GPIO.OUT)

    def write_byte(self, byte):
        """Writes an 8-bit value to the shift register."""
        GPIO.output(self.latch_pin, 0)
        for i in range(8):
            GPIO.output(self.clock_pin, 0)
            GPIO.output(self.data_pin, (byte >> (7 - i)) & 1)  # MSB first
            GPIO.output(self.clock_pin, 1)
        GPIO.output(self.latch_pin, 1)
        self.current_byte = byte  # Update the internal state

    def clear(self):
        """Clears the shift register (sets all outputs to 0)."""
        self.write_byte(0)

    def set_pin(self, pin, state):
        """
        Sets an individual pin to HIGH (1) or LOW (0).
        Pins are numbered 0-7, with 0 being the MSB.
        """
        if not 0 <= pin <= 7:
            raise ValueError("Pin must be in range 0-7.")
        if state:
            self.current_byte |= (1 << (7 - pin))  # Set the bit to 1
        else:
            self.current_byte &= ~(1 << (7 - pin))  # Set the bit to 0
        self.write_byte(self.current_byte)

    def toggle_pin(self, pin):
        """Toggles the state of an individual pin."""
        if not 0 <= pin <= 7:
            raise ValueError("Pin must be in range 0-7.")
        self.current_byte ^= (1 << (7 - pin))  # Flip the bit
        self.write_byte(self.current_byte)
